"""Plugins Handler

Module realizes plugins handler.
"""
from sanic.response import HTTPResponse

from ...web.handlers import BaseHandler


class PluginsHandler(BaseHandler):
    """
    Handler for plugins list
    """

    async def get(self) -> HTTPResponse:
        """
        Get list of application plugins

        .. _spec:
            _static/api.html#operation/plugins

        Resource is reached by address '/plugins'

        """
        allPlugins = []
        for pluginClasses in self.request.appCtx.pluginManager.importedPlugins.values():
            allPlugins.extend(pluginClass.__name__ for pluginClass in pluginClasses)
        activePlugins = [plugin.pluginName for plugin in self.request.appCtx.pluginManager.activePlugins]
        plugins = []
        for plugin in allPlugins:
            plugins.append({"name": plugin, "running": 1 if plugin in activePlugins else 0})
        return self.success(statusCode=200, outputJson={"plugins": plugins})
